#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

from coopr.pyomo.base import IPyomoPresolver, IPyomoPresolveAction, _VarBase
import pyutilib.plugin.core

ignore = set(['"', ' ', "'", '`'])
#underscore = set(['[', ']', "(", ')', ',', '-'])
underscore = set(['-'])

class GenericVarLabelsPresolver(pyutilib.plugin.core.SingletonPlugin):
    """
    This plugin changes variables names to generate a label that
    can be used in LP/MILP file formats.
    """

    pyutilib.plugin.core.implements(IPyomoPresolveAction)

    def __init__(self, **kwds):
        kwds['name'] = "generic_varlabels"
        pyutilib.plugin.core.Plugin.__init__(self, **kwds)

    def rank(self):
        return 1000

    def _name_fix(self, name):
        global ignore
        global underscore
        tmp=""
        for c in name:
            if c in underscore:
                tmp += '_'
            elif c is '(':
                tmp += '['
            elif c is ')':
                tmp += ']'
            elif not c in ignore:
                tmp += c
        return tmp

    def presolve(self,model):
        """
        The main routine to perform the presolve
        """
        #
        # This creates labels for all variables, not just the ones used
        # in this instance.  This is useful when interrogating unused 
        # variables.
        #
        Vars = model._component[_VarBase]
        for var in Vars.values():
            for V in var._varval.keys():
                var._varval[V].label = self._name_fix( var._varval[V].name )
                model._name_varmap[ var._varval[V].label ] = var._varval[V]
        for id in model._con:
            model._con[id].label = self._name_fix( model._con[id].name )
        return model
            
